@@ -9,7 +9,7 @@
 #
 #  KeepNote
 #  Copyright (c) 2008-2011 Matt Rasmussen
-#  Author: Matt Rasmussen <rasmus@mit.edu>
+#  Author: Matt Rasmussen <rasmus@alum.mit.edu>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -58,7 +58,6 @@ from keepnote.gui.icons import \
     get_node_icon, lookup_icon_filename
 from keepnote.gui.font_selector import FontSelector
 from keepnote.gui.colortool import FgColorTool, BgColorTool
-from keepnote.gui.richtext.richtext_tags import color_tuple_to_string
 from keepnote.gui.popupwindow import PopupWindow
 from keepnote.gui.linkcomplete import LinkPickerPopup
 from keepnote.gui.link_editor import LinkEditor
@@ -66,6 +65,7 @@ from keepnote.gui.editor import KeepNoteEditor
 from keepnote.gui import \
     CONTEXT_MENU_ACCEL_PATH, \
     DEFAULT_FONT, \
+    DEFAULT_COLORS, \
     FileChooserDialog, \
     get_pixbuf, \
     get_resource, \
@@ -254,7 +254,11 @@ class RichTextEditor (KeepNoteEditor):
         else:
             # no new notebook, clear the view
             self.clear_view()
-        
+
+    def get_notebook(self):
+        """Returns notebook"""
+        return self._notebook
+
     
     def load_preferences(self, app_pref, first_open=False):
         """Load application preferences"""
@@ -263,15 +267,24 @@ class RichTextEditor (KeepNoteEditor):
             app_pref.get("editors", "general", "spell_check",
                          default=True))
 
+        try:
+            format = app_pref.get("editors", "general", "quote_format")
+            self._textview.set_quote_format(format)
+        except:
+            pass
+
         self.load_notebook_preferences()
 
 
     def save_preferences(self, app_pref):
         """Save application preferences"""
-
+        
         # record state in preferences
         app_pref.set("editors", "general", "spell_check", 
                      self._textview.is_spell_check_enabled())
+        app_pref.set("editors", "general", "quote_format", 
+                     self._textview.get_quote_format())
+
 
     def load_notebook_preferences(self):
         """Load notebook-specific preferences"""
@@ -305,19 +318,19 @@ class RichTextEditor (KeepNoteEditor):
         """Redo the last action in the viewer"""
         self._textview.redo()
     
-    def view_pages(self, pages):
+    def view_nodes(self, nodes):
         """View a page in the editor"""
         
-        # editor cannot view multiple pages at once
+        # editor cannot view multiple nodes at once
         # if asked to, it will view none
-        if len(pages) > 1:
-            pages = []
+        if len(nodes) > 1:
+            nodes = []
 
-        # save current page before changing pages
+        # save current page before changing nodes
         self.save()
         self._save_cursor()
 
-        pages = [node for node in pages
+        pages = [node for node in nodes
                  if node.get_attr("content_type") ==
                     notebooklib.CONTENT_TYPE_PAGE]
         
@@ -329,7 +342,9 @@ class RichTextEditor (KeepNoteEditor):
             self._page = page
             self._textview.enable()
             
-            try:
+            try:                
+                self._textview.set_current_url(page.get_url(), 
+                                               title=page.get_title())
                 self._textview_io.set_node(self._page)
                 self._textview_io.load(
                     self._textview,
@@ -985,20 +1000,29 @@ class EditorMenus (gobject.GObject):
         
         if color == 0:
             color = widget.color            
-
-        if color is not None:
-            colorstr = color_tuple_to_string(color)
-        else:
-            colorstr = None
-
+        
         if kind == "fg":
-            self._editor.get_textview().set_font_fg_color(colorstr)
+            self._editor.get_textview().set_font_fg_color(color)
         elif kind == "bg":
-            self._editor.get_textview().set_font_bg_color(colorstr)
+            self._editor.get_textview().set_font_bg_color(color)
         else:
             raise Exception("unknown color type '%s'" % str(kind))
+
+
+    def _on_colors_set(self, colors):
+        """Set color pallete"""
+
+        # save colors
+        notebook = self._editor._notebook
+        if notebook:
+            notebook.pref.set("colors", list(colors))
+            notebook.set_preferences_dirty()
+
+        self._app.get_listeners("colors_changed").notify(notebook, colors)
         
 
+
+
     def _on_choose_font(self):
         """Callback for opening Choose Font Dialog"""
         
@@ -1072,7 +1096,6 @@ class EditorMenus (gobject.GObject):
         # remove action group
         window.get_uimanager().remove_action_group(self._action_group)
         self._action_group = None
-        
 
 
     def get_actions(self):
@@ -1409,6 +1432,16 @@ class EditorMenus (gobject.GObject):
                 widget = ui.widget.get_proxies()[0]
                 widget.set_active(active)
 
+        def replace_widget(path, widget):
+            w = uimanager.get_widget(path)
+            if w:
+                self._removed_widgets.append(w.child)
+                w.remove(w.child)
+                w.add(widget)
+                widget.show()
+                w.set_homogeneous(False)
+
+
         self.setup_font_toggle(
             uimanager, "/main_tool_bar/Viewer/Editor/Bold Tool", 
             update_func=lambda ui, font: update_toggle(ui, font.mods["bold"]))
@@ -1452,31 +1485,20 @@ class EditorMenus (gobject.GObject):
             uimanager, "/main_tool_bar/Viewer/Editor/Bullet List Tool", 
             update_func=lambda ui, font:
             update_toggle(ui, font.par_type == "bullet"))
-        #lambda ui, font:
-                #ui.widget.set_active(font.par_type == "bullet"))
-
-
         
+
         # family combo
         font_family_combo = FontSelector()
         font_family_combo.set_size_request(150, 25)
-
-
-        # TODO: make proper custom tools
-        
-        w = uimanager.get_widget("/main_tool_bar/Viewer/Editor/Font Selector Tool")
-        if w:
-            self._removed_widgets.append(w.child)
-            w.remove(w.child)
-            w.add(font_family_combo)
-            font_family_combo.show()
-            font_family_id = font_family_combo.connect("changed",
-                                                       self._on_family_set)
-            self._font_ui_signals.append(
-                FontUI(font_family_combo,
-                       font_family_id,
-                       update_func=lambda ui, font: 
-                       ui.widget.set_family(font.family)))
+        replace_widget("/main_tool_bar/Viewer/Editor/Font Selector Tool",
+                       font_family_combo)
+        font_family_id = font_family_combo.connect("changed",
+                                                   self._on_family_set)
+        self._font_ui_signals.append(
+            FontUI(font_family_combo,
+                   font_family_id,
+                   update_func=lambda ui, font: 
+                   ui.widget.set_family(font.family)))
 
         # font size
         DEFAULT_FONT_SIZE = 10
@@ -1486,55 +1508,60 @@ class EditorMenus (gobject.GObject):
         font_size_button.set_size_request(-1, 25)
         font_size_button.set_value(DEFAULT_FONT_SIZE)
         font_size_button.set_editable(False)
+        replace_widget("/main_tool_bar/Viewer/Editor/Font Size Tool",
+                       font_size_button)
+
+        font_size_id = font_size_button.connect(
+            "value-changed",
+            lambda w: 
+            self._on_font_size_change(font_size_button.get_value()))
+        self._font_ui_signals.append(
+            FontUI(font_size_button,
+                   font_size_id,
+                   update_func=lambda ui, font:
+                       ui.widget.set_value(font.size)))
+
+
+        def on_new_colors(notebook, colors):
+            if self._editor.get_notebook() == notebook:
+                self.fg_color_button.set_colors(colors)
+                self.bg_color_button.set_colors(colors)
+        self._app.get_listeners("colors_changed").add(on_new_colors)
         
-        w = uimanager.get_widget("/main_tool_bar/Viewer/Editor/Font Size Tool")
-        if w:
-            self._removed_widgets.append(w.child)
-            w.remove(w.child)
-            w.add(font_size_button)
-            font_size_button.show()
-            w.set_homogeneous(False)
-            font_size_id = font_size_button.connect("value-changed",
-                lambda w: 
-                self._on_font_size_change(font_size_button.get_value()))
-            self._font_ui_signals.append(
-                FontUI(font_size_button,
-                       font_size_id,
-                       update_func=lambda ui, font:
-                           ui.widget.set_value(font.size)))
+
+        # init colors
+        notebook = self._editor.get_notebook()
+        if notebook:
+            colors = notebook.pref.get("colors", default=DEFAULT_COLORS)
+        else:
+            colors = DEFAULT_COLORS        
 
 
         # font fg color
         # TODO: code in proper default color
-        self.fg_color_button = FgColorTool(14, 15, (0, 0, 0))
+        self.fg_color_button = FgColorTool(14, 15, "#000000")
+        self.fg_color_button.set_colors(colors)
         self.fg_color_button.set_homogeneous(False)
         self.fg_color_button.connect("set-color",
             lambda w, color: self._on_color_set(
                 "fg", self.fg_color_button, color))
-
-        w = uimanager.get_widget("/main_tool_bar/Viewer/Editor/Font Fg Color Tool")
-        if w:
-            self._removed_widgets.append(w.child)
-            w.remove(w.child)
-            w.add(self.fg_color_button)
-            self.fg_color_button.show()
-            w.set_homogeneous(False)
+        self.fg_color_button.connect("set-colors",
+            lambda w, colors: self._on_colors_set(colors))
+        replace_widget("/main_tool_bar/Viewer/Editor/Font Fg Color Tool",
+                       self.fg_color_button)
 
         # font bg color
-        self.bg_color_button = BgColorTool(14, 15, (65535, 65535, 65535))
+        self.bg_color_button = BgColorTool(14, 15, "#ffffff")
+        self.bg_color_button.set_colors(colors)
         self.bg_color_button.set_homogeneous(False)
         self.bg_color_button.connect(
             "set-color",
             lambda w, color: self._on_color_set(
                 "bg", self.bg_color_button, color))
-
-        w = uimanager.get_widget("/main_tool_bar/Viewer/Editor/Font Bg Color Tool")
-        if w:
-            self._removed_widgets.append(w.child)
-            w.remove(w.child)
-            w.add(self.bg_color_button)
-            self.bg_color_button.show()
-            w.set_homogeneous(False)
+        self.bg_color_button.connect("set-colors",
+            lambda w, colors: self._on_colors_set(colors))
+        replace_widget("/main_tool_bar/Viewer/Editor/Font Bg Color Tool",
+                       self.bg_color_button)
 
 
         # get spell check toggle
