@@ -7,8 +7,8 @@
 
 #
 #  KeepNote
-#  Copyright (c) 2008-2009 Matt Rasmussen
-#  Author: Matt Rasmussen <rasmus@mit.edu>
+#  Copyright (c) 2008-2011 Matt Rasmussen
+#  Author: Matt Rasmussen <rasmus@alum.mit.edu>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,147 +38,260 @@ from keepnote.gui import \
      get_resource, \
      get_resource_image, \
      get_resource_pixbuf
+from keepnote.gui.icons import get_node_icon
 from keepnote.gui import treemodel
 from keepnote import notebook
 from keepnote.notebook import NoteBookError
 import keepnote
+import keepnote.timestamp
 
 _ = keepnote.translate
 
 
+DEFAULT_ATTR_COL_WIDTH = 150
+DEFAULT_TITLE_COL_WIDTH = 250
+
+
 
 class KeepNoteListView (basetreeview.KeepNoteBaseTreeView):
     
     def __init__(self):
         basetreeview.KeepNoteBaseTreeView.__init__(self)
         self._sel_nodes = None
+        self._columns_set = False
+        self._current_table = "default"
+        self._col_widths = {}
+        self.time_edit_format = "%Y/%m/%d %H:%M:%S"
 
         # configurable callback for setting window status
         self.on_status = None        
         
-        # init model
-        self.set_model(gtk.TreeModelSort(treemodel.KeepNoteTreeModel()))
-
         # selection config
         self.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
-
-
-        self.model.connect("sort-column-changed", self._sort_column_changed)
         
         # init view
         self.connect("key-release-event", self.on_key_released)
         self.connect("button-press-event", self.on_button_press)
         self.connect("row-expanded", self._on_listview_row_expanded)
         self.connect("row-collapsed", self._on_listview_row_collapsed)
-        
+        self.connect("columns-changed", self._on_columns_changed)
+
         self.set_rules_hint(True)
         self.set_fixed_height_mode(True)
-        
-        
-        # title column
-        cell_icon = gtk.CellRendererPixbuf()
-        self.title_text = gtk.CellRendererText()
-        self.title_column = gtk.TreeViewColumn()
-        self.title_column.set_title(_("Title"))
-        self.title_column.set_property("sizing", gtk.TREE_VIEW_COLUMN_FIXED)
-        self.title_column.set_min_width(10)
-        self.title_column.set_fixed_width(250)
-        self.title_column.set_property("resizable", True)
-        self.title_column.pack_start(cell_icon, False)
-        self.title_column.pack_start(self.title_text, True)
-        #self.title_column.connect("clicked", self.on_column_clicked)
-        self.title_text.set_fixed_height_from_font(1)
-        self.title_text.connect("edited", self.on_edit_title)
-        self.title_text.connect("editing-started", self.on_editing_started)
-        self.title_text.connect("editing-canceled", self.on_editing_canceled)        
-        self.title_text.set_property("editable", True)
-        self.title_column.set_sort_column_id(
-            self.rich_model.get_column_by_name("title_sort").pos)
-        # map cells to columns in model
-        self.title_column.add_attribute(cell_icon, 'pixbuf',
-            self.rich_model.get_column_by_name("icon").pos)
-        self.title_column.add_attribute(cell_icon, 'pixbuf-expander-open',
-            self.rich_model.get_column_by_name("icon_open").pos)
-        self.title_column.add_attribute(self.title_text, 'text',
-            self.rich_model.get_column_by_name("title").pos)
-        self.append_column(self.title_column)
-        self.set_expander_column(self.title_column)
-        
-        
-        # created column
-        cell_text = gtk.CellRendererText()
-        cell_text.set_fixed_height_from_font(1)        
-        column = gtk.TreeViewColumn()
-        column.set_title(_("Created"))
-        column.set_property("sizing", gtk.TREE_VIEW_COLUMN_FIXED)
-        column.set_property("resizable", True)
-        column.set_min_width(10)
-        column.set_fixed_width(150)
-        column.set_sort_column_id(
-            self.rich_model.get_column_by_name("created_time_sort").pos)
-        #column.connect("clicked", self.on_column_clicked)
-        #column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
-        #column.set_property("min-width", 5)
-        column.pack_start(cell_text, True)
-        column.add_attribute(cell_text, 'text',
-            self.rich_model.get_column_by_name("created_time").pos)
-        self.append_column(column)
-    
-        # modified column
-        cell_text = gtk.CellRendererText()
-        cell_text.set_fixed_height_from_font(1)
-        column = gtk.TreeViewColumn()
-        column.set_title(_("Modified"))
-        column.set_property("sizing", gtk.TREE_VIEW_COLUMN_FIXED)
-        column.set_property("resizable", True)
-        column.set_min_width(10)
-        column.set_fixed_width(150)
-        column.set_sort_column_id(
-            self.rich_model.get_column_by_name("modified_time_sort").pos)
-        #column.connect("clicked", self.on_column_clicked)
-        #column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
-        #column.set_property("min-width", 5)
-        column.pack_start(cell_text, True)
-        column.add_attribute(cell_text, 'text',
-            self.rich_model.get_column_by_name("modified_time").pos)
-        self.append_column(column)
-        
-        
-        # set default sorting
-        # remember sort per node
-        self.model.set_sort_column_id(
-            self.rich_model.get_column_by_name("order").pos,
-            gtk.SORT_ASCENDING)
-        self.set_reorder(basetreeview.REORDER_ALL)
-        
         self.set_sensitive(False)
+                
 
-
+        # init model
+        self.set_model(gtk.TreeModelSort(treemodel.KeepNoteTreeModel()))
+        
+        self.setup_columns()
     
+    
+    def load_preferences(self, app_pref, first_open=False):
+        """Load application preferences"""
+        self.set_date_formats(app_pref.get("timestamp_formats"))
+        self.set_rules_hint(
+            app_pref.get("look_and_feel", "listview_rules",
+                         default=True))
+
+
+    def save_preferences(self, app_pref):
+        """Save application preferences"""
+        pass                
+
+
     def set_notebook(self, notebook):
         """Set the notebook for listview"""
+        
+        if notebook != self._notebook and self._notebook is not None:
+            self._notebook.get_listeners("table_changed").remove(
+                self._on_table_changed)
+
         basetreeview.KeepNoteBaseTreeView.set_notebook(self, notebook)
         
         if self.rich_model is not None:
             self.rich_model.set_root_nodes([])
 
         if notebook:
+            # load notebook prefs
             self.set_sensitive(True)
+            notebook.get_listeners("table_changed").add(self._on_table_changed)
         else:
             self.set_sensitive(False)
 
+        self.setup_columns()
 
-    def set_date_formats(self, formats):
-        """Set the date formats used for dates"""
-        if self.rich_model:
-            self.rich_model.set_date_formats(formats)
 
-    def get_root_nodes(self):
-        """Returns the root nodes displayed in listview"""
-        if self.rich_model:
-            return self.rich_model.get_root_nodes()
+    def save(self):
+        """Save notebook preferences"""
+        
+        if self._notebook is None:
+            return
+
+        self._save_column_widths()
+        self._save_column_order()
+
+        self._notebook.mark_modified()
+
+
+    def _save_column_widths(self):
+
+        # save attr column widths
+        widths = self._notebook.get_attr("column_widths", {})
+        for col in self.get_columns():
+            widths[col.attr] = col.get_width()
+        self._notebook.set_attr("column_widths", widths)
+        
+
+    def _save_column_order(self):
+
+        # save column attrs
+        table = self._notebook.attr_tables.get(self._current_table)
+        table.attrs = [col.attr for col in self.get_columns()]
+        
+        # TODO: notify table change
+
+
+    def _load_column_widths(self):
+
+        widths = self._notebook.get_attr("column_widths", {})
+        for col in self.get_columns():
+            width = widths.get(col.attr, DEFAULT_ATTR_COL_WIDTH)
+            if col.get_width() != width and width > 0:
+                col.set_fixed_width(width)
+                widths[col.attr] = width
+        
+
+    def _load_column_order(self):
+
+        current_attrs = [col.attr for col in self.get_columns()]
+
+        table = self._notebook.attr_tables.get(self._current_table)
+        
+        if table.attrs != current_attrs:
+            if set(current_attrs) == set(table.attrs):
+                # only order changed
+                lookup = dict((col.attr, col) for col in self.get_columns())
+                prev = None
+                for attr in table.attrs:
+                    col = lookup[attr]
+                    self.move_column_after(col, prev)
+                    prev = col
+            else:
+                # resetup all columns
+                self.setup_columns()
+            
+
+    def _on_table_changed(self, notebook, table):
+        
+        if self._notebook == notebook and table == self._current_table:
+            self._load_column_widths()
+            self._load_column_order()
+
+
+    #==================================
+    # model and view setup
+
+
+    def set_model(self, model):
+        basetreeview.KeepNoteBaseTreeView.set_model(self, model)
+        self.model.connect("sort-column-changed", self._sort_column_changed)
+
+
+    def setup_columns(self):
+        
+        self.clear_columns()
+        
+        if self._notebook is None:
+            self._columns_set = False
+            return
+        
+        # TODO: eventually columns may change when ever master node changes
+        attrs = self._notebook.attr_tables.get(self._current_table).attrs
+        
+        
+        # add columns        
+        for attr in attrs:
+            col = self._add_column(attr)
+            col.set_reorderable(True) # allow column reordering
+            if attr == self._attr_title:
+                self.title_column = col
+
+        # add model columns
+        self._add_model_column("order")
+        
+        # NOTE: must create a new TreeModelSort whenever we add new columns
+        # to the rich_model that could be used in sorting
+        # Perhaps something is being cached
+        self.set_model(gtk.TreeModelSort(self.rich_model))
+        
+        
+        # config columns view
+        self.set_expander_column(self.get_column(0))
+
+        # TODO: load correct sorting right away
+        # set default sorting
+        # remember sort per node
+        self.model.set_sort_column_id(
+            self.rich_model.get_column_by_name("order").pos,
+            gtk.SORT_ASCENDING)
+        self.set_reorder(basetreeview.REORDER_ALL)
+
+        self._columns_set = True
+        
+
+    def _add_column(self, attr, cell_attr=None):
+        
+        # get attribute definition from notebook
+        attr_def = self._notebook.attr_defs.get(attr)
+
+        # get datatype
+        if attr_def is not None:
+            datatype = attr_def.datatype
+            col_title = attr_def.name
         else:
-            return []
+            datatype = "string"
+            col_title = attr
+
+        # get/make model column
+        self._add_model_column(attr)
+
+        # create column view
+        column = gtk.TreeViewColumn()
+        column.attr = attr
+        column.set_property("sizing", gtk.TREE_VIEW_COLUMN_FIXED)
+        column.set_property("resizable", True)
+        column.connect("notify::width", self._on_column_width_change)
+        column.set_min_width(10)
+        column.set_fixed_width(
+            self._notebook.get_attr("column_widths", {}).get(
+                attr, DEFAULT_ATTR_COL_WIDTH))
+        column.set_title(col_title)
+
+        # define column sorting
+        attr_sort = attr + "_sort"
+        col =  self.rich_model.get_column_by_name(attr_sort)
+        if col:
+            column.set_sort_column_id(col.pos)
+
+        # add cell renders
+        if attr == self._attr_title:
+            self._add_title_render(column, attr)
+        elif datatype == "timestamp":
+            self._add_text_render(column, attr, editable=True,
+                                  validator=basetreeview.TextRendererValidator(
+                    lambda x: keepnote.timestamp.format_timestamp(
+                        x, self.time_edit_format),
+                    lambda x: keepnote.timestamp.parse_timestamp(
+                        x, self.time_edit_format)))
+        else:
+            self._add_text_render(column, attr)
+
+
+        self.append_column(column)
+
+        return column
+
 
     #=============================================
     # gui callbacks    
@@ -208,7 +321,7 @@ class KeepNoteListView (basetreeview.KeepNoteBaseTreeView):
         else:
             col = self.rich_model.get_column(col_id)
         
-        if col is None or col.attr == "order":
+        if col is None: #  or col.attr == "order"
             self.model.set_sort_column_id(
                 self.rich_model.get_column_by_name("order").pos,
                 gtk.SORT_ASCENDING)
@@ -221,7 +334,7 @@ class KeepNoteListView (basetreeview.KeepNoteBaseTreeView):
         """Callback for key release events"""
 
         # no special processing while editing nodes
-        if self.editing:
+        if self.editing_path:
             return
 
         if event.keyval == gtk.keysyms.Delete:
@@ -266,6 +379,34 @@ class KeepNoteListView (basetreeview.KeepNoteBaseTreeView):
 
         # TODO: document this more
         return self.get_master_node() is not None
+
+
+    def _on_columns_changed(self, treeview):
+        """Callback for when columns change order"""
+
+        if not self._columns_set:
+            return
+
+        # config columns view
+        col = self.get_column(0)
+        if col:
+            self.set_expander_column(col)
+
+        if self._notebook:
+            self._save_column_order()
+            self._notebook.get_listeners("table_changed").notify(
+                self._notebook, self._current_table)
+
+    def _on_column_width_change(self, col, width):
+
+        width = col.get_width()
+        if (self._notebook and 
+            self._col_widths.get(col.attr, None) != width):
+            self._col_widths[col.attr] = width
+            self._save_column_widths()
+            self._notebook.get_listeners("table_changed").notify(
+                self._notebook, self._current_table)
+        
     
     #====================================================
     # actions
@@ -313,6 +454,15 @@ class KeepNoteListView (basetreeview.KeepNoteBaseTreeView):
         self.emit("select-nodes", [])
 
 
+
+    def get_root_nodes(self):
+        """Returns the root nodes displayed in listview"""
+        if self.rich_model:
+            return self.rich_model.get_root_nodes()
+        else:
+            return []
+    
+
     def append_node(self, node):
 
         # do not allow appending of nodes unless we are masterless
@@ -334,7 +484,6 @@ class KeepNoteListView (basetreeview.KeepNoteBaseTreeView):
     def display_page_count(self, npages=None):
 
         if npages is None:
-            #npages = len(self.get_root_nodes())
             npages = self.count_pages(self.get_root_nodes())
 
         if npages != 1:
@@ -350,7 +499,7 @@ class KeepNoteListView (basetreeview.KeepNoteBaseTreeView):
         def walk(node):
             npages = 1
             if (self.rich_model.get_nested() and 
-                (node.get_attr("expanded2"))):
+                (node.get_attr("expanded2", False))):
                 for child in node.get_children():
                     npages += walk(child)
             return npages
@@ -402,8 +551,8 @@ class KeepNoteListView (basetreeview.KeepNoteBaseTreeView):
     def load_sorting(self, node, model):
         """Load sorting information from node"""
 
-        info_sort = node.get_attr("info_sort")
-        sort_dir = node.get_attr("info_sort_dir")
+        info_sort = node.get_attr("info_sort", "order")
+        sort_dir = node.get_attr("info_sort_dir", 1)
             
         if sort_dir:
             sort_dir = gtk.SORT_ASCENDING
@@ -414,8 +563,9 @@ class KeepNoteListView (basetreeview.KeepNoteBaseTreeView):
         if info_sort == "":
             info_sort = "order"
 
+        # TODO: do not rely on *_sort convention
         for col in self.rich_model.get_columns():
-            if info_sort == col.attr:
+            if info_sort == col.attr and col.name.endswith("_sort"):
                 model.set_sort_column_id(col.pos, sort_dir)
 
         self._update_reorder()
@@ -446,3 +596,6 @@ class KeepNoteListView (basetreeview.KeepNoteBaseTreeView):
         self.display_page_count()
 
 
+
+
+
