@@ -7,8 +7,8 @@
 
 #
 #  KeepNote
-#  Copyright (c) 2008-2009 Matt Rasmussen
-#  Author: Matt Rasmussen <rasmus@mit.edu>
+#  Copyright (c) 2008-2011 Matt Rasmussen
+#  Author: Matt Rasmussen <rasmus@alum.mit.edu>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -37,7 +37,6 @@ import urllib2
 import uuid
 
 # xml imports
-from xml.sax.saxutils import escape
 import xml.etree.cElementTree as ET
 
 
@@ -61,7 +60,8 @@ from keepnote.notebook import sync
 
 _ = trans.translate
 
-
+#=============================================================================
+# constants
 
 # NOTE: the <?xml ?> header is left off to keep it compatiable with IE,
 # for the time being.
@@ -316,6 +316,18 @@ def write_empty_page(node, page_file=PAGE_DATA_FILE):
 #=============================================================================
 # adding content to a notebook/nodes
 
+
+def new_page(parent, title=None, index=None):
+    """Add a new page to a node in a notebook"""
+
+    if title is None:
+        title = DEFAULT_PAGE_NAME
+
+    child = parent.new_child(CONTENT_TYPE_PAGE, title, index)
+    write_empty_page(child)
+    return child
+
+
 def attach_file(filename, node, index=None):
     """Attach a file to a node in a notebook"""
 
@@ -341,16 +353,6 @@ def attach_file(filename, node, index=None):
         raise e
 
 
-def new_page(parent, title=None, index=None):
-    """Add a new page to a node in a notebook"""
-
-    if title is None:
-        title = DEFAULT_PAGE_NAME
-
-    child = parent.new_child(CONTENT_TYPE_PAGE, title, index)
-    write_empty_page(child)
-    return child
-
 
 
 #=============================================================================
@@ -388,112 +390,194 @@ class NoteBookVersionError (NoteBookError):
 #=============================================================================
 # notebook attributes
 
+_datatype_defaults = {
+    "string": "",
+    "integer": 0,
+    "float": 0.0,
+    "bool": False}
 
 class AttrDef (object):
     """
-    A AttrDef is a metadata attribute that can be associated to
-    nodes in a NoteBook.
+    An AttrDef defines the type of an notebook attr
     """
 
     def __init__(self, key, datatype, name, default=None):
 
-        self.name = name
-        self.datatype = datatype
         self.key = key
+        self.datatype = datatype
+        self.name = name
 
-        # default function
+        # default value
         if default is None:
-            self.default = datatype
+            self.default = _datatype_defaults.get(datatype, None)
         else:
             self.default = default
-        
-        # writer function
-        if datatype == bool:
-            self.write = lambda x: unicode(int(x))
-        else:
-            self.write = unicode
 
-        # reader function
-        if datatype == bool:
-            self.read = lambda x: bool(int(x))
-        else:
-            self.read = datatype
 
-        
+    def format(self):
+        """
+        Returns dict representation
+        """
 
-class UnknownAttr (object):
-    """A value that belongs to an unknown AttrDef"""
+        return {"key": self.key,
+                "datatype": self.datatype,
+                "name": self.name,
+                "default": self.default}
 
-    def __init__(self, value):
-        self.value = value
 
+class AttrDefs (object):
+    """ 
+    A collection of AttrDef's
+    """
+
+    def __init__(self):
+        self._attr_defs = {}
+
+    def add(self, attr_def):
+        self._attr_defs[attr_def.key] = attr_def
+
+    def remove(self, key):
+        if key in self._attr_defs:
+            del self._attr_defs[key]
+
+    def clear(self):
+        self._attr_defs.clear()
+
+    def get(self, key):
+        return self._attr_defs.get(key, None)
+
+    def parse(self, lst):
+        for item in lst:
+            self.add(parse_attr_def(item))
+
+    def format(self):
+        return [attr_def.format()
+                for attr_def in self._attr_defs.itervalues()]
+
+
+
+def format_attr_def(attr_def):
+    return attr_def.format()
+
+
+def parse_attr_def(attr_def_dict):
+    return AttrDef(attr_def_dict["key"],
+                   attr_def_dict["datatype"],
+                   attr_def_dict.get("name", attr_def_dict["key"]),
+                   default=attr_def_dict.get("default", None))
+
+def iter_attr_defs(lst):
+    for item in lst:
+        yield parse_attr_def(item)
+
+
+# typedef timestamp integer
 
 g_default_attr_defs = [
-    AttrDef("nodeid", unicode, "Node ID", default=new_nodeid),
-    AttrDef("content_type", unicode, "Content type", 
-            default=lambda: CONTENT_TYPE_DIR),
-    AttrDef("title", unicode, "Title"),
-    AttrDef("order", int, "Order", default=lambda: sys.maxint),
-    AttrDef("created_time", int, "Created time", default=get_timestamp),
-    AttrDef("modified_time", int, "Modified time", default=get_timestamp),
-    AttrDef("expanded", bool, "Expaned", default=lambda: True),
-    AttrDef("expanded2", bool, "Expanded2", default=lambda: True),
-    AttrDef("info_sort", unicode, "Folder sort", default=lambda: "order"),
-    AttrDef("info_sort_dir", int, "Folder sort direction", default=lambda: 1),
-    AttrDef("icon", unicode, "Icon"),
-    AttrDef("icon_open", unicode, "Icon open"),
-    AttrDef("payload_filename", unicode, "Filename"),
-    AttrDef("duplicate_of", unicode, "Duplicate of")
+    AttrDef("nodeid", "string", "Node ID"),
+    AttrDef("content_type", "string", "Content type", default=CONTENT_TYPE_DIR),
+    AttrDef("title", "string", "Title"),
+    AttrDef("order", "integer", "Order", default=sys.maxint),
+    AttrDef("created_time", "timestamp", "Created time"),
+    AttrDef("modified_time", "timestamp", "Modified time"),
+    AttrDef("expanded", "bool", "Expaned", default=True),
+    AttrDef("expanded2", "bool", "Expanded2", default=True),
+    AttrDef("info_sort", "string", "Folder sort", default="order"),
+    AttrDef("info_sort_dir", "integer", "Folder sort direction", default=1),
+    AttrDef("icon", "string", "Icon"),
+    AttrDef("icon_open", "string", "Icon open"),
+    AttrDef("payload_filename", "string", "Filename"),
+    AttrDef("duplicate_of", "string", "Duplicate of"),
+    AttrDef("title_bgcolor", "string", "Title Background Color")
 ]
 
 
-'''
-class NoteBookTable (object):
-    def __init__(self, name, attrs=[]):
+
+class AttrTable (object):
+    def __init__(self, key, name, attrs=[]):
+        self.key = key
         self.name = name
         self.attrs = list(attrs)
-
-        # TODO: add col widths
-        # NoteBooks have tables and attrs
-
-default_notebook_table = NoteBookTable("default", attrs=[title_attr,
-                                                         created_time_attr,
-                                                         modified_time_attr])
-'''
+        
+    def format(self):
+        return {"key": self.key,
+                "name": self.name,
+                "attrs": list(self.attrs)}
 
 
-# TODO: parent might be an implict attr
+class AttrTables (object):
+    """ 
+    A collection of AttrTable's
+    """
+
+    def __init__(self):
+        self._attr_tables = {}
+
+    def add(self, attr_table):
+        self._attr_tables[attr_table.key] = attr_table
+
+    def remove(self, key):
+        if key in self._attr_tables:
+            del self._attr_tables[key]
+
+    def clear(self):
+        self._attr_tables.clear()
+
+    def get(self, key):
+        return self._attr_tables.get(key, None)
+
+    def parse(self, lst):
+        for item in lst:
+            self.add(parse_attr_table(item))
+
+    def format(self):
+        return [attr_table.format()
+                for attr_table in self._attr_tables.itervalues()]
 
 
-# 1. attrs should be data that is optional (although keepnote has a few
-# required entries).
-# 2. attrs can appear in listview
+
+g_default_attr_tables = [
+    AttrTable(
+        "default", "Default Table", 
+        attrs=["title", "created_time", "modified_time"])]
 
 
+def format_attr_table(attr_table):
+    return attr_table.format()
+
+
+def parse_attr_table(dct):
+    return AttrTable(dct["key"], dct["name"], dct["attrs"])
+
+
+def iter_attr_tables(lst):
+    for item in lst:
+        yield parse_attr_table(item)
 
 
 #=============================================================================
 # Notebook nodes
 
+BUILTIN_ATTR = ("nodeid", "parentids", "childrenids", "order")
+
 class NoteBookNode (object):
     """A general base class for all nodes in a NoteBook"""
 
     def __init__(self, title=u"", parent=None, notebook=None,
                  content_type=CONTENT_TYPE_DIR, conn=None,
-                 init_attr=True):
+                 attr=None):
         self._notebook = notebook
         self._conn = conn if conn else self._notebook._conn
         self._parent = parent
         self._children = None
         self._has_children = None
         self._valid = True
-        self._attr = {}
-        
-        if init_attr:
-            self.clear_attr(title=title, content_type=content_type)
 
-        # TODO: add a mechanism to register implict attrs that in turn do lookup
-        # "parent", "nchildren"
+        self._attr = {"version": NOTEBOOK_FORMAT_VERSION,
+                      "title": title,
+                      "content_type": content_type}
+        if attr:
+            self._attr.update(attr)
         
         
     def is_valid(self):
@@ -530,14 +614,10 @@ class NoteBookNode (object):
     
     def clear_attr(self, title="", content_type=CONTENT_TYPE_DIR):
         """Clear attributes (set them to defaults)"""
+        for key in self._attr.keys():
+            if key not in BUILTIN_ATTR:
+                del self._attr[key]
         
-        # TODO: get keys from a default attr list of some sort
-        self._attr.clear()
-        keys = ["order", "expanded", "expanded2", 
-                "info_sort", "info_sort_dir", "created_time", "modified_time"]
-        for key in keys:
-            self._attr[key] = self._notebook.attr_defs[key].default()
-
         # set title and content_type
         self._attr["title"] = title
         self._attr["content_type"] = content_type
@@ -547,18 +627,15 @@ class NoteBookNode (object):
         """Get the value of an attribute"""
         return self._attr.get(name, default)
 
+
     def set_attr(self, name, value):
         """Set the value of an attribute"""
         oldvalue = self._attr.get(name, NULL)
         self._attr[name] = value
-
-        # if attr is one that the notebook manages then we are dirty
-        # TODO: should have additional test that attr needs to be saved
-        # this test is added for the icon_loaded attr, which is not needed to
-        # to be saved
-        if name in self._notebook.attr_defs and value != oldvalue:
+        if value != oldvalue:
             self._set_dirty(True)
 
+
     def has_attr(self, name):
         """Returns True if node has the attribute"""
         return name in self._attr
@@ -570,9 +647,7 @@ class NoteBookNode (object):
         # TODO: check against un-deletable attributes
         if name in self._attr:
             del self._attr[name]
-
-        if name in self._notebook.attr_defs:
-            self._set_dirty(True)
+        self._set_dirty(True)
         
 
     def iter_attr(self):
@@ -580,12 +655,14 @@ class NoteBookNode (object):
         return self._attr.iteritems()
     
 
-    def _init_attr(self, attr):
+    def _init_attr(self):
         """Initialize attributes from a dict"""
-        attr.setdefault("version", NOTEBOOK_FORMAT_VERSION)
-        if self._notebook.set_attr_defaults(attr):
-            self._set_dirty(True)
-        self._attr.update(attr)
+        t = get_timestamp()
+        self._attr.setdefault("created_time", t)
+        self._attr.setdefault("modified_time", t)
+        self._attr.setdefault("childrenids", [])
+        self._attr.setdefault("parentids", [])
+
 
     #========================================
     # special attr methods
@@ -650,13 +727,14 @@ class NoteBookNode (object):
     def create(self):
         """Initializes the node on disk (create required files/directories)"""
 
-        self._attr["created_time"] = get_timestamp()
-        self._attr["modified_time"] = get_timestamp()
-        self._attr["parentids"] = [self._parent._attr["nodeid"]]
-        self._attr["childrenids"] = []
         if "nodeid" not in self._attr:
             self._attr["nodeid"] = new_nodeid()
-
+        self._attr["parentids"] = [self._parent._attr["nodeid"]]
+        self._attr["childrenids"] = []
+        self._attr.setdefault("order", sys.maxint)
+        
+        self._init_attr()
+        
         self._conn.create_node(self._attr["nodeid"], self._attr)
         self._set_dirty(False)
        
@@ -664,6 +742,11 @@ class NoteBookNode (object):
     def delete(self):
         """Deletes this node from the notebook"""
 
+        # when deleting multiple nodes, some nodes (children) might already
+        # be invalid.  Therefore, ignore them.
+        if not self._valid:
+            return
+
         # check whether this is allowed
         allowed, error = self._notebook.delete_allowed(self)
         if not allowed:
@@ -731,6 +814,7 @@ class NoteBookNode (object):
         
         # TODO: if parent is in another notebook, accessory data like icons 
         # might need to be transferred.
+        #  Maybe I won't support that.
 
         # TODO: with multiple parents, we need to specify here which 
         # parent relationship we are breaking.
@@ -752,7 +836,7 @@ class NoteBookNode (object):
         # make sure new parents children are loaded
         parent.get_children()
 
-        # perform on-disk move if new parent
+        # perform actual move if new parent
         if old_parent != parent:
             try:
                 self._attr["parentids"] = [parent._attr["nodeid"]]
@@ -761,7 +845,7 @@ class NoteBookNode (object):
                 self._attr["parentids"] = [old_parent._attr["nodeid"]]
                 raise
 
-        # perform move in data structure
+        # perform move in NoteBook data structure
         self._parent._remove_child(self)
         if self._parent != parent:
             self._parent._set_child_order()
@@ -821,30 +905,19 @@ class NoteBookNode (object):
             keepnote.log_error()
             pass
 
-
+    
     def rename(self, title):
         """Renames the title of the node"""
         
-        # do nothing if title is the same
-        if title == self._attr["title"]:
-            return
-        
-        if self._parent is None:
-            # don't rename the directory of the notebook itself
-            # just change the title
-            self._attr["title"] = title
-            self._set_dirty(True)
-        else:
-            oldtitle = self._attr.get("title", "")
-            try:
-                self._attr["title"] = title
-                self.save(True)
-            except NoteBookError, e:
-                self._attr["title"] = oldtitle
-                raise
-        
-        self.notify_change(False)
+        # TODO: keep attach file names in sync
 
+        self.set_attr("title", title)
+        self.save()
+
+        # TODO: generalize notify for all attrs that might appear in listview
+        # or other GUI widgets
+        self.notify_change(False)
+        
 
     def new_child(self, content_type, title, index=None):
         """Add a new node under this node"""
@@ -925,9 +998,9 @@ class NoteBookNode (object):
 
     def _write_attr(self, attr):
         
-        self._notebook._mask_attr.set_dict(attr)
-        self._conn.update_node(attr["nodeid"], self._notebook._mask_attr)
-        #self._conn.update_node(attr["nodeid"], attr)
+        #self._notebook._mask_attr.set_dict(attr)
+        #self._conn.update_node(attr["nodeid"], self._notebook._mask_attr)
+        self._conn.update_node(attr["nodeid"], attr)
 
 
     #==================================
@@ -977,26 +1050,19 @@ class NoteBookNode (object):
            Returns temporary node objects
         """
         
-        default_content_type = CONTENT_TYPE_DIR
-        
-        for childid in self._attr["childrenids"]:
+        for childid in self._attr["childrenids"]: 
             try:
-                attr = self._conn.read_node(childid)
+                yield self._notebook._read_node(childid, parent=self)
             except:
                 keepnote.log_error()
                 continue
-            node = NoteBookNode(
-                attr.get("title", DEFAULT_PAGE_NAME), 
-                parent=self, notebook=self._notebook,
-                content_type=attr.get("content_type", default_content_type))
-            node._init_attr(attr)
-            yield node
+
     
     
     def _set_child_order(self):
         """Ensures that child know their order in the children list"""
         for i, child in enumerate(self._children):
-            if child._attr["order"] != i:
+            if child._attr.get("order") != i:
                 child._attr["order"] = i
                 child._set_dirty(True)
         
@@ -1193,49 +1259,61 @@ class NoteBook (NoteBookNode):
         
         self._conn = None # Note: this comes first in order satify base class
         NoteBookNode.__init__(self, notebook=self, 
-                              content_type=CONTENT_TYPE_DIR,
-                              init_attr=False)
+                              content_type=CONTENT_TYPE_DIR)
         
         self.pref = NoteBookPreferences()
         self._filename = None
         self._dirty = set()
         self._trash = None
-        self.attr_defs = {}
+        self.attr_defs = AttrDefs()
+        self.attr_tables = AttrTables()
         self._necessary_attrs = []
         
-        self._mask_attr = maskdict.MaskDict(
-            {}, ["icon_open_load", "icon_load"])
-        
-
         # init notebook attributes
         self._init_default_attr()
-        self.clear_attr()
 
         self._attr["order"] = 0
-        self._attr["title"] = ""
-
 
         # listeners
+        self.listeners = {}
         self.node_changed = Listeners()  # signature = (node, recurse)
         self.closing_event = Listeners()
         self.close_event = Listeners()
 
 
+    def get_listeners(self, key):
+        """Get custom listener"""
+        listeners = self.listeners.get(key, None)
+        if listeners is None:
+            listeners = Listeners()
+            self.listeners[key] = listeners
+        return listeners
+
+
     #=====================================
     # attrs        
 
     def _init_default_attr(self):
         """Initialize default notebook attributes"""
         
-        self._necessary_attrs = ["nodeid", "created_time", "modified_time"]
-        self.clear_attr_defs()
-        for attr in g_default_attr_defs:
-            self.add_attr_def(attr)
-        
+        # TODO: not being used right now
+        self._necessary_attrs = ["nodeid", "created_time", "modified_time",
+                                 "order"]
+
+        # init attr defs
+        self.attr_defs.clear()
+        for attr_def in g_default_attr_defs:
+            self.attr_defs.add(attr_def)
+
+        # init attr tables
+        self.attr_tables.clear()
+        for attr_table in g_default_attr_tables:
+            self.attr_tables.add(attr_table)
+
 
     def add_attr_def(self, attr_def):
         """Adds a new attribute definition to the notebook"""
-        self.attr_defs[attr_def.key] = attr_def
+        self.attr_defs.add(attr_def)
     
 
     def clear_attr_defs(self):
@@ -1248,15 +1326,17 @@ class NoteBook (NoteBookNode):
         return self._necessary_attrs
 
 
-    def set_attr_defaults(self, attr):
-        """Set default attributes in an attr dict"""
-        modified = False
-        for key in self._necessary_attrs:
-            if key not in attr:
-                attr[key] = self.attr_defs[key].default()
-                modified = True
-        return modified
+    def _read_attr_defs(self):
+        self._init_default_attr()
+        
+        self.attr_defs.parse(self._attr.get("attr_defs", ()))
+        self.attr_tables.parse(self._attr.get("attr_tables", ()))
 
+
+    def _write_attr_defs(self):
+        self._attr["attr_defs"] = self.attr_defs.format()
+        self._attr["attr_tables"] = self.attr_tables.format()
+    
     
     #===================================================
     # input/output
@@ -1267,9 +1347,9 @@ class NoteBook (NoteBookNode):
         self._conn = conn if conn else connection_fs.NoteBookConnectionFS()
         self._filename = filename
 
-        self._attr["created_time"] = get_timestamp()
-        self._attr["modified_time"] = get_timestamp()
         self._attr["nodeid"] = new_nodeid()
+        self._init_attr()
+
 
         self._conn.connect(filename)
         self._conn.create_node(self._attr["nodeid"],  self._attr)
@@ -1301,7 +1381,7 @@ class NoteBook (NoteBookNode):
             if os.path.exists(pref_file):
                 try:
                     self.read_preferences(
-                        safefile.open(pref_file, codec="utf-8"), recover=False)
+                        safefile.open(pref_file, codec="utf-8"))
 
                     # TODO: temp solution. remove soon.
                     index_dir = self.pref.get("index_dir", default=u"")
@@ -1310,6 +1390,12 @@ class NoteBook (NoteBookNode):
                             os.path.join(index_dir, notebook_index.INDEX_FILE))
                 except:
                     pass
+
+            # check version
+            version = get_notebook_version(filename)
+            if version > NOTEBOOK_FORMAT_VERSION:
+                raise NoteBookVersionError(version, NOTEBOOK_FORMAT_VERSION)
+
         
         # read basic info
         self._filename = filename
@@ -1317,10 +1403,13 @@ class NoteBook (NoteBookNode):
         self._init_index()
 
         attr = self._conn.read_node(self._conn.get_rootid())
-        self._init_attr(attr)
+        self._attr.update(attr)
+        self._init_attr()
 
         self._init_trash()
 
+        self._read_attr_defs()
+
         self.read_preferences()
 
         self.notify_change(True)
@@ -1333,6 +1422,7 @@ class NoteBook (NoteBookNode):
         # TODO: keepnote copy of old pref.  only save pref if its changed.
 
         if force or self in self._dirty:
+            self._write_attr_defs()
             self._write_attr(self._attr)
             #self._conn.update_node(self._attr["nodeid"], self._attr)
             self.write_preferences()
@@ -1391,8 +1481,8 @@ class NoteBook (NoteBookNode):
         
         node = NoteBookNode(attr.get("title", DEFAULT_PAGE_NAME), 
                             parent=parent, notebook=self,
-                            content_type=content_type)
-        node._init_attr(attr)
+                            content_type=content_type, 
+                            attr=attr)
         node.create()
         return node
 
@@ -1436,17 +1526,17 @@ class NoteBook (NoteBookNode):
         return node in self._dirty
 
 
-    def _read_node(self, nodeid, parent=None, attr=None, 
+    def _read_node(self, nodeid, parent=None, 
                    default_content_type=CONTENT_TYPE_DIR):
 
-        if attr is None:
-            attr = self._conn.read_node(nodeid)
-        
+        attr = self._conn.read_node(nodeid)
         node = NoteBookNode(
             attr.get("title", DEFAULT_PAGE_NAME), 
             parent=parent, notebook=self,
-            content_type=attr.get("content_type", default_content_type))
-        node._init_attr(attr)
+            content_type=attr.get("content_type", default_content_type),
+            attr=attr)
+        node._init_attr()
+
         return node
 
     #=====================================
@@ -1604,7 +1694,7 @@ class NoteBook (NoteBookNode):
 
         path = self._conn.get_node_path_by_id(nodeid)
         if path is None:
-            keepnote.log_message("node %s not found" % nodeid)
+            keepnote.log_message("node %s not found\n" % nodeid)
             return None
         
         def walk(node, path, i):
@@ -1618,7 +1708,7 @@ class NoteBook (NoteBookNode):
                     return walk(child, path, i+1)
             
             # node not found
-            keepnote.log_message("node %s not found" % str(path))
+            keepnote.log_message("node %s not found\n" % str(path))
             return None
         return walk(self._notebook, path[1:], 0)
     
@@ -1636,12 +1726,19 @@ class NoteBook (NoteBookNode):
 
     def has_fulltext_search(self):
         """Returns True if full text indexed search is availble"""
-        return self._conn.has_fulltext_search()
+        return self._conn.index(["has_fulltext"])
+
+    def enable_fulltext_search(self, enabled):
+        """Returns True if full text indexed search is availble"""
+        return self._conn.index(["enable_fulltext", enabled])
 
     def get_attr_by_id(self, nodeid, key):
         """Returns attr value for a node with id 'nodeid'"""
         return self._conn.get_attr_by_id(nodeid, key)
     
+    def index(self, query):
+        return self._conn.index(query)
+
 
     #----------------------------------------
     # index interface (temparary until fully transparent)
@@ -1687,11 +1784,11 @@ class NoteBook (NoteBookNode):
         """Writes the NoteBooks preferences"""
         try:
             # ensure preference directory exists
-            self._conn.create_dir(self._attr["nodeid"], NOTEBOOK_META_DIR)
+            self._conn.create_dir(self._attr["nodeid"], NOTEBOOK_META_DIR + "/")
                 
             # ensure icon directory exists
             self._conn.create_dir(self._attr["nodeid"], 
-                             NOTEBOOK_META_DIR + "/" +  NOTEBOOK_ICON_DIR)
+                             NOTEBOOK_META_DIR + "/" +  NOTEBOOK_ICON_DIR + "/")
 
             data = self.pref.get_data()
 
@@ -1711,7 +1808,7 @@ class NoteBook (NoteBookNode):
             raise NoteBookError(_("File format error"), e)
 
     
-    def read_preferences(self, infile=None, recover=True):
+    def read_preferences(self, infile=None):
         """Reads the NoteBook's preferneces"""
         
         try:
@@ -1724,12 +1821,12 @@ class NoteBook (NoteBookNode):
                                 % self.get_file(PREF_FILE) , e)
         except Exception, e:
             keepnote.log_error(e)
-            if recover:
-                if infile:
-                    infile.close()
-                    infile = None
-                self._recover_preferences()
-                return self.read_preferences(recover=False)
+            #if recover:
+            #    if infile:
+            #        infile.close()
+            #        infile = None
+            #    self._recover_preferences()
+            #    return self.read_preferences(recover=False)
             raise NoteBookError(_("Notebook preference data is corrupt"), e)
         finally:
             if infile:
